/*
Copyright 2007 Daniel Gonzalez Peña, Florentino Fernandez Riverola


This file is part of the AIBench Project. 

AIBench Project is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

AIBench Project is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser Public License for more details.

You should have received a copy of the GNU Lesser Public License
along with AIBench Project.  If not, see <http://www.gnu.org/licenses/>.
 */

/*  
 * SplashFrame.java
 * Created inside the SING research group (http://sing.ei.uvigo.es)
 * University of Vigo
 *
 */

package es.uvigo.ei.aibench;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.GraphicsDevice.WindowTranslucency;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import org.apache.log4j.Logger;
import org.platonos.pluginengine.Plugin;
import org.platonos.pluginengine.event.IPluginEngineListener;
import org.platonos.pluginengine.event.PluginEngineEvent;
import org.platonos.pluginengine.event.PluginEngineEventType;

public class SplashFrame extends JFrame implements IPluginEngineListener {
	
	private static Logger		logger				= Logger.getLogger(SplashFrame.class);
	
	private static final long	serialVersionUID	= 1L;
	
	private JPanel				jContentPane		= null;
	
	private JProgressBar		progress			= new JProgressBar();
	
	private Image				fImage;
	
	private int					current				= 0, toStart = 0;
	
	private boolean				translucent			= false;
	
	public SplashFrame() {
		super();
		initialize();
		
		this.progress.setStringPainted(true);
		this.progress.setBackground(new Color(255, 255, 255, 0));
		this.progress.setBorderPainted(false);
		this.progress.setAlignmentX(JProgressBar.LEFT_ALIGNMENT);
		this.progress.setMaximum(20);
		this.progress.setString("Initializing plugins...");
	}
	
	private void initialize() {
		GraphicsDevice gd = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
		translucent = gd.isWindowTranslucencySupported(WindowTranslucency.PERPIXEL_TRANSLUCENT);
		
		if (Launcher.CONFIG.containsKey("application.icon.16") && Launcher.CONFIG.containsKey("application.icon.32")) {
			List<Image> icons = new ArrayList<Image>();
			ImageIcon icon16 = new ImageIcon(Launcher.CONFIG.getProperty("application.icon.16"));
			icons.add(icon16.getImage());
			ImageIcon icon32 = new ImageIcon(Launcher.CONFIG.getProperty("application.icon.32"));
			icons.add(icon32.getImage());
			setIconImages(icons);
		}
		this.setLayout(new BorderLayout());
		this.add(getJContentPane(), BorderLayout.NORTH);
		this.setSize(fImage.getWidth(null), fImage.getHeight(null) + 10);
		this.add(progress, BorderLayout.SOUTH);
		this.setTitle("Welcome to AIBench");
		
		// Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		// Rectangle window = getBounds();
		// setLocation((screen.width - window.width) / 2, (screen.height -
		// window.height) / 2);
		
		this.setAlwaysOnTop(true);
		this.setUndecorated(true);
		// this.setOpacity(opacity)
		if (translucent)
			this.setBackground(new Color(255, 255, 255, 0));
		else
			this.setBackground(new Color(255, 255, 255, 255));
		
		this.pack();
		
		this.setLocationRelativeTo(null);
	}
	
	private javax.swing.JPanel getJContentPane() {
		if (jContentPane == null) {
			
			URL imageURL = null;
			if (Launcher.CONFIG.getProperty("splashimage") != null) {
				imageURL = Launcher.class.getProtectionDomain().getCodeSource().getLocation();
				try {
					if (imageURL.getFile().endsWith(".jar")) {
						imageURL = new URL(imageURL.toString().substring(0, imageURL.toString().lastIndexOf('/')) + "/../" + Launcher.CONFIG.getProperty("splashimage"));
					} else {
						imageURL = new URL(imageURL + "../" + Launcher.CONFIG.getProperty("splashimage"));
					}
					
				} catch (MalformedURLException e1) {
					System.err.println("Not found the specified splash image, searching in url: " + imageURL.getFile() + "using default");
					imageURL = SplashFrame.class.getResource("/splash.jpg");
				}
				
			} else {
				imageURL = SplashFrame.class.getResource("/splash.jpg");
			}
			
			// fImage = Toolkit.getDefaultToolkit().getImage(imageURL);
			MediaTracker mt = new MediaTracker(this);
			fImage = Toolkit.getDefaultToolkit().getImage(imageURL);
			
			mt.addImage(fImage, 0);
			
			try {
				mt.waitForAll();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			
			jContentPane = new javax.swing.JPanel() {
				
				private static final long	serialVersionUID	= 1L;
				
				public void paintComponent(Graphics g) {
					super.paintComponent(g);
					
					if (fImage != null) {
						int imwidth = fImage.getWidth(null);
						int imheight = fImage.getHeight(null);
						
						if ((imwidth > 0) && (imheight > 0)) {
							for (int y = 0; y < getHeight(); y += imheight) {
								for (int x = 0; x < getWidth(); x += imwidth) {
									g.drawImage(fImage, x, y, null);
								}
							}
						}
					}
				}
			};
			jContentPane.setLayout(null);
			jContentPane.setOpaque(true);
			if (translucent)
				jContentPane.setBackground(new Color(255, 255, 255, 0));
			else
				jContentPane.setBackground(new Color(255, 255, 255, 0));
			
			jContentPane.setPreferredSize(new java.awt.Dimension(fImage.getWidth(null), fImage.getHeight(null)));
		}
		
		return jContentPane;
	}
	
	public void handlePluginEngineEvent(PluginEngineEvent evt) {
		if (evt.getEventType().equals(PluginEngineEventType.STARTUP)) {
			
		} else if (evt.getEventType().equals(PluginEngineEventType.PLUGIN_LOADED)) {
			Plugin plugin = (Plugin) evt.getPayload();
			if (!plugin.isDisabled() && plugin.getStartWhenResolved()) {
				this.toStart++;
				this.progress.setMaximum(this.toStart);
			}
			logger.info("Loaded " + plugin.getName() + "   [version " + plugin.getVersion() + "]");
		} else if (evt.getEventType().equals(PluginEngineEventType.PLUGIN_STARTING)) {
			this.progress.setString("Starting " + evt.getPayload().toString());
		} else if (evt.getEventType().equals(PluginEngineEventType.PLUGIN_STARTED) || evt.getEventType().equals(PluginEngineEventType.PLUGIN_DISABLED)) {
			this.progress.setValue(++this.current);
			
			if (this.toStart == this.current) {
				this.setVisible(false);
				this.dispose();
			}
		}
	}
	
	static public void main(String... strings) throws InterruptedException {
		SplashFrame teste = new SplashFrame();
		teste.setVisible(true);
		float num = 0.01f;
		float current = 1f;
		while (current > 0.0001f) {
			System.out.println("current = " + current);
			current -= num;
			Thread.sleep(50);
			teste.setOpacity(current);
		}
		teste.dispose();
	}
}
